@@ -1,13 +1,10 @@
-#!/usr/bin/python
 #
-# Copyright (c) 2017-2019 NVIDIA CORPORATION. All rights reserved.
+# Copyright (c) 2017-2021 NVIDIA CORPORATION. All rights reserved.
 # This file is part of the WebDataset library.
 # See the LICENSE file for licensing terms (BSD-style).
 #
 
-
-"""A small wrapper for the functions in the `iterators` package
-transforming them into curried iterators."""
+"""A small curry wrapper for the functions in the `iterators` package."""
 
 
 from . import iterators
@@ -16,36 +13,43 @@ from . import iterators
 class Curried2(object):
     """Helper class for currying pipeline stages.
 
-    We use this roundabout construct becauce it can be pickled."""
+    We use this roundabout construct becauce it can be pickled.
+    """
 
     def __init__(self, f, *args, **kw):
+        """Create a curried function."""
         self.f = f
         self.args = args
         self.kw = kw
 
     def __call__(self, data):
+        """Call the curried function with the given argument."""
         return self.f(data, *self.args, **self.kw)
 
     def __str__(self):
+        """Compute a string representation."""
         return f"<{self.f.__name__} {self.args} {self.kw}>"
 
     def __repr__(self):
+        """Compute a string representation."""
         return f"<{self.f.__name__} {self.args} {self.kw}>"
 
 
 class Curried(object):
     """Helper class for currying pipeline stages.
 
-    We use this roundabout construct because it can be pickled."""
+    We use this roundabout construct because it can be pickled.
+    """
 
     def __init__(self, f):
+        """Store the function for future currying."""
         self.f = f
 
     def __call__(self, *args, **kw):
+        """Curry with the given arguments."""
         return Curried2(self.f, *args, **kw)
 
 
-map_stream = Curried(iterators.map_stream)
 info = Curried(iterators.info)
 shuffle = Curried(iterators.shuffle)
 select = Curried(iterators.select)
